unit SudoCase;

interface

uses
  SysUtils, Classes,dialogs,Types, Controls,ExtCtrls,Graphics,math;

Type
  // composant TSudoCase (Instanci 81 fois pour generer une grille )

  TSudoCase = class(TShape)
  private
    FRegion          : Integer;
    FColonne         : Integer;
    FLigne           : Integer;
    FCandidats       : String[9];
    FNumber          : String;
    FViewCandidat    : Boolean;
    FonMouseDown     : TMouseEvent;
    FCouleurchiffre  : Integer;
    FCouleurCandidat : Integer;
    FCouleurFond     : Integer;

    procedure SetColonne(const Value: Integer);
    procedure SetLigne(const Value: Integer);
    procedure SetNumber(const Value: String);
    procedure SetCouleurChiffre(const Value: integer);
    procedure SetCouleurCandidat(const Value: integer);
    procedure SetCouleurFond(const Value: integer);
    procedure SetViewCandidat(const Value: Boolean);
    Procedure AppelSelecteur(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);



  private
  { Dclarations prives }

  protected
    { Dclarations protges }
   procedure Paint; override;
   procedure DessinCandidat(const Value: Integer);
   // renvoi a zone (ou region) de la case en fonction de sa ligne et colonne
   function  RenvoiZone(Ligne,Colonne : Integer): Integer ;

  public
    { Dclarations publiques }
   constructor Create(AOwner: TComponent);override;
   Procedure   Vider;
   procedure   ClearCandidat(const Value: Integer);

   Function  ExistCandidat(const Value: Integer):Boolean;
   Function  CandidatPossible:Boolean;
   Function  RenvoiCandidats : String;
   Function  CompterFlag():Integer;

  published
    { Dclarations publies }
    Property Colonne         : Integer  read FColonne write SetColonne;
    Property Ligne           : Integer  read FLigne   write SetLigne;
    Property Region          : Integer  read FRegion  ;
    Property Number          : String   read FNumber  write SetNumber;
    Property CouleurChiffre  : Integer read FCouleurChiffre  Write SetCouleurChiffre;
    Property CouleurCandidat : Integer read FCouleurCandidat Write SetCouleurCandidat;
    Property CouleurFond     : Integer read FCouleurfond     Write SetCouleurFond;
    Property ViewCandidat    : Boolean read FViewCandidat    Write SetViewCandidat;

  end;


implementation



{ TSudoCase }

constructor TSudoCase.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 //Taille, Forme et texte
 Width      := 39;
 Height     := 39;
 Shape      := stRoundRect;
 FCouleurFond := ClWhite;
 FCandidats :='TTTTTTTTT'; // 9 fois True au depart tout est possible
 inherited OnMouseDown := AppelSelecteur; //C'est appelSelecteur qui sera appel en cas OnMouseDown}
end;

// Appel du code Mouse Down pour le composant
// implementation faite dans le parent
Procedure TSudoCase.AppelSelecteur(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
Begin
 IF Assigned(FOnMouseDown) then FOnMouseDown(self,Button,Shift,X,Y);
End;

////////////////////////////
/// les initialisations  ///
////////////////////////////

// Attribution de la valeur du n de Colonne
procedure TSudoCase.SetColonne(const Value: Integer);
begin
  FColonne := Value;
  FRegion := RenvoiZone(Fligne,FColonne);
end;

// Attribution de la valeur du n de Ligne
procedure TSudoCase.SetLigne(const Value: Integer);
begin
  FLigne := Value;
  FRegion := RenvoiZone(Fligne,FColonne);
end;

// Rengion le N de region (ou zone)
function TSudoCase.RenvoiZone(Ligne,Colonne : Integer): Integer ;
begin
 Result := 0;
 If Ligne  = 0 then exit;
 If Colonne = 0 then exit;
 Result := Ceil(Colonne/3)+((ceil(Ligne/3)-1)*3);
end;

/////////////////////
/// les asseceurs ///
/////////////////////

// Propriet boolenne T/F pour voir ou cacher les candidats
procedure TSudoCase.SetViewCandidat(const Value: Boolean);
begin
 if FViewCandidat <> Value then
 begin
  FViewCandidat := Value;
  Paint;
 end;
end;

// Propriet modifiant la couleur du chiffre
procedure TSudoCase.SetCouleurChiffre(const Value: integer);
begin
 if FCouleurChiffre <> Value then
 begin
  FCouleurChiffre := Value;
  Paint;
 end;
end;

// Propriet modifiant la couleur des candidats
procedure TSudoCase.SetCouleurCandidat(const Value: integer);
begin
 If FCouleurCandidat <> Value then
 Begin
  FCouleurCandidat := Value;
  Paint;
 End;
end;

// Propriet modifiant la couleur du fond de la case
procedure TSudoCase.SetCouleurFond(const Value: integer);
begin
 if FCouleurfond <> Value then
 begin
  FCouleurfond := Value;
  Paint;
 end;
end;

/////////////////////////////
/// gestion des candidats ///
/////////////////////////////

// Test l'existance d'un Flag  pour le candidat
Function TSudoCase.ExistCandidat(const Value: Integer):Boolean;
Begin
 Result := False;
 IF FCandidats[Value] = 'T' then
  Result :=True;
End;

// Efface un candidat existant
procedure TSudoCase.ClearCandidat(const Value: Integer);
begin
 IF not ExistCandidat(Value) then
  Exit;
 FCandidats[Value] := 'F';
 paint;
end;

// compte le nombre de candidat existant
Function TSudoCase.CompterFlag():Integer;
Var
 I : Integer;
Begin
 Result := 0;
 For I := 1 TO 9 Do
  IF FCandidats[I] = 'T' then
   Inc(Result);
End;

// Renvoi une chaine contenant les candidats possibles
Function  TSudoCase.RenvoiCandidats():String;
Var
 I : Integer;
Begin
 Result := EmptyStr;
 For I := 1 TO 9 Do
  IF FCandidats[I] = 'T' then
  Result := Result +IntToStr(i);
End;

// Pose le chiffre dans la case
procedure TSudoCase.SetNumber(const Value: String);
Var
 I : Integer;
begin
 IF Value <> FNumber then
 Begin
  //Mettre les flags candidats = 'F'
  For I := 1 To 9 Do
    ClearCandidat(I);
  FNumber := Value;
 End;
 Paint;
end;

// precise si il reste des candidats possibles
Function TSudoCase.CandidatPossible:Boolean;
Var
 I : Integer;
 J : Integer;
Begin
 J := 0 ;
 Result := False;
 For I := 1 TO 9 Do
  IF FCandidats[I] = 'T' then
   Inc(J);
 If J = 1 then Result := True;
End;

// tout propre
Procedure TSudoCase.Vider;
begin
 FCandidats :='TTTTTTTTT';
 FNumber := EmptyStr;
 Paint;
end;

///////////////
/// dessins ///
///////////////

// Paint surcharg
procedure TSudoCase.Paint;
Var
 I : Integer;
Begin
 Inherited paint;
 // carr blanc
 Canvas.Brush.Color := CouleurFond;
 Canvas.Brush.Style := bsSolid;
 Canvas.FloodFill(2,2,Clwhite,fsSurface);
 If FNumber <> EmptyStr then // dessiner le chiffre
 begin
  Canvas.Font.Size  := 15;
  Canvas.Font.Name  := 'Times New Roman';
  Canvas.Font.Color := FCouleurchiffre;
  Canvas.TextOut(14,9,FNumber);
 end
 else
 IF FViewCandidat = True then // si les candidats doivent tre dessin
  For I := 1 To 9 Do DessinCandidat(I); // appel de la routine de dessin
End;

// Dessin d'un candidat dans la case
procedure TSudoCase.DessinCandidat(const Value: Integer);
begin
 If FCandidats[Value] = 'F' then
  Exit;
 //dessin
 Canvas.Font.Size  := 8;
 Canvas.Font.Name  := 'Courrier New';
 Canvas.Font.Color := FcouleurCandidat ;
 case value of
  1 : Canvas.TextOut(6,2,'1');
  2 : Canvas.TextOut(17,2,'2');
  3 : Canvas.TextOut(29,2,'3');
  4 : Canvas.TextOut(6,13,'4');
  5 : Canvas.TextOut(17,13,'5');
  6 : Canvas.TextOut(29,13,'6');
  7 : Canvas.TextOut(6,24,'7');
  8 : Canvas.TextOut(17,24,'8');
  9 : Canvas.TextOut(29,24,'9');

 end;
// IF Value = 1 then  Canvas.TextOut(6,2,'1');
// IF Value = 2 then Canvas.TextOut(17,2,'2');
// IF Value = 3 then Canvas.TextOut(29,2,'3');
// IF Value = 4 then Canvas.TextOut(6,13,'4');
// IF Value = 5 then Canvas.TextOut(17,13,'5');
// IF Value = 6 then Canvas.TextOut(29,13,'6');
// IF Value = 7 then Canvas.TextOut(6,24,'7');
// IF Value = 8 then Canvas.TextOut(17,24,'8');
// IF Value = 9 then Canvas.TextOut(29,24,'9');
End;


end.
