unit UcreerGrille;

interface

uses UAnalyse,math,SysUtils;
    var
    GrilleGen     : TGrille;
    LalisteGen    : TListDec;
    PosteCourant  : BYTE;

    Procedure InitGrille;
    function  Generer: String;
    procedure Methode1;
    procedure Methode2;
    Procedure MettreAjourCases(Ligne,Colonne : ShortInt;Valeur : String);
    procedure MettreUnDevoile;
    procedure LaCaseAjour(Ligne,Colonne : ShortInt;Valeur : String);
    function  Aleatoire(Num1,Num2:integer):integer;
    Function  TestSiFini:   boolean;
    Function  TestSiErreur: boolean;
    Function  CreerChaine:  String;
    function  RenvoiRegion(Ligne,Colonne : Integer): Integer;
    function  SupprimerCandidat(LaChaine,Lecandidat : String):String;

implementation
///*******************************************************
/// cette unit genere des grilles de sudoku
/// la gnration n'est pas aletaoire
///  le principe est le suivant
///  Mettre tout les candidats de chaque case  A POSSIBLE
///  1/ rechercher  au hasard une case vierge
///  2/ rechercher au hasard un candidat encore possible pour cette case
///  3/ poser le chiffre et suprimer ce chiffre de la liste des candidats en L,Cet region
///    3.1 marque ce chiffre comme chiffre Devoil
///  4/ tester si il y a des incoherence (case vierge sans candidats par exemple)
///  5/ si erreur detecte tout vider repartir en 1
///  6/ Jouer la grille avec 2 mthodes de resolution (les plus simples)
///     6.1 marquer le ou les chiffres decouverts comme  jouer (en oposition  3.1)
///  7/ Tester si 81 cases trouve si oui Grille OK on sort si non retour en 1
///
/// les cases nomm Devoil sont les chiffres de dpart de la grille
/// cet algo genere environ 350 Grilles jouables /secondes

///*******************************************************


Function Generer: string;
var
 I        : Integer;
 Fini     : Boolean;
 Erreur   : Boolean;
begin
 Randomize;
 repeat
  InitGrille;// initialisation de la grille
  For I:= 1 to 17 do
   MettreUnDevoile; // 17 cases pour commencer
  repeat
   MettreUnDevoile;
   Methode1;
   Methode2;
   Fini    := TestSiFini;
   Erreur  := TestSiErreur;
  Until Erreur or Fini;
 Until Fini;
 result := creerChaine;
End;

procedure MettreUnDevoile;
var
 A,B : Integer;
begin
 Repeat
  A := Aleatoire(1,81);
 until (GrilleGen[A].Valeur = '.');
 B := Aleatoire(1,length(GrilleGen[A].Cand));
 GrilleGen[A].Valeur  := GrilleGen[A].Cand[B];
 GrilleGen[A].Cand    := EmptyStr;
 GrilleGen[A].Devoile := 1;
 MettreAjourCases(GrilleGen[A].Ligne,GrilleGen[A].Colonne,GrilleGen[A].Valeur);
end;

Function  TestSiFini:boolean;
var
 I : Integer;
begin
 result := True; // on y croit !!!
 For I := 1 to 81 do
  If GrilleGen[I].Valeur = '.' then
  begin
   result := False; // False
   exit;
  end;
end;

Function  TestSiErreur:boolean;
var
 I : Integer;
begin
 result := False; // pas d'erreur on y croit !!!
 For I := 1 to 81 do
  If (GrilleGen[I].cand = EmptyStr) and(GrilleGen[I].valeur = '.') then
  begin
   result := True; // y'a une erreur
   exit;
  end;
end;

//****************************//
// initialisation de la grille
//****************************//
procedure InitGrille;
Var
 caseCourante,Ligne,Colonne :ShortInt;
begin
 For caseCourante := 0 to 80 do
  begin
   Ligne   :=   succ(caseCourante div 9);
   Colonne :=   succ(caseCourante mod 9);
   GrilleGen[caseCourante+1].Ligne   := Ligne;
   GrilleGen[caseCourante+1].Colonne := Colonne;
   GrilleGen[caseCourante+1].Region  := RenvoiRegion(Ligne,Colonne);
   GrilleGen[caseCourante+1].Valeur  := '.';
   GrilleGen[caseCourante+1].LigReg  := PlaceEnregion(Ligne);
   GrilleGen[caseCourante+1].ColReg  := PlaceEnregion(Colonne);
   GrilleGen[caseCourante+1].Cand    := '123456789';
   GrilleGen[caseCourante+1].devoile := 0;
  end;
end;

// ******************************
// pose un devoil
// *****************************
Procedure MettreAjourCases(Ligne,Colonne : ShortInt;Valeur : String);
Var
 I :Integer;
begin
 For I := 1 to 81 do
 Begin
  If GrilleGen[I].Ligne = ligne then
    GrilleGen[I].Cand := SupprimerCandidat(GrilleGen[I].Cand,Valeur);
  If GrilleGen[I].Colonne = Colonne then
    GrilleGen[I].Cand := SupprimerCandidat(GrilleGen[I].Cand,Valeur);
  If GrilleGen[I].region = RenvoiRegion(Ligne,Colonne) then
    GrilleGen[I].Cand := SupprimerCandidat(GrilleGen[I].Cand,Valeur);
 End;
end;


function SupprimerCandidat(LaChaine,Lecandidat : String):String;
Var
 I : Integer;
begin
 Result := EmptyStr;
 For I := 1 To length(LaChaine) do
  If LaChaine[I] <> Lecandidat then
    Result := Result + LaChaine[I];
End;


// ******************************
// Recuperer chaine
// ******************************
Function CreerChaine: String;
Var
 I : Integer;
begin
 Result := EmptyStr;
 For I := 1 to 81 do
 If GrilleGen[I].Devoile = 1 then
  Result := Result + GrilleGen[I].valeur
 else
  Result := Result + '.';
end;



//************************************************//
// renvoi la region d'une case situe en ligne
// et colonne
//************************************************//
function RenvoiRegion(Ligne,Colonne : Integer): Integer ;
begin
 Result := Ceil(Colonne/3)+((ceil(Ligne/3)-1)*3);
end;

//************************************************//
// renvoi un nombre aleatoire compris entre
// Num1 et Num2     exemple : Aleatoire(1,81)
//************************************************//
function Aleatoire(Num1,Num2:integer):integer;
var
 Diff: integer;
begin
 inc(Num2);
 Diff:=Num2-Num1;
 Result := Random(Diff)+ Num1;
end;

  //***************************************//
  // les methodes de gnration de grilles //
  //***************************************//

//***************************************//
// la premiere mthode recherche les cases
// ou seul 1 candidat reste possible
//***************************************//
procedure Methode1;
Var
 I      : Integer;
 Trouve : Boolean;
begin
 Trouve := False;
 For I := 1 to 81 do
   If length(GrilleGen[I].Cand) = 1 then
   begin
    GrilleGen[I].Valeur  := GrilleGen[I].Cand;
    GrilleGen[I].Devoile := 0;
    GrilleGen[I].Cand    := EmptyStr;
    MettreAjourCases(GrilleGen[I].ligne,GrilleGen[I].Colonne,GrilleGen[I].Valeur);
    Trouve := True;
   end;
 // Si au moins une case a t trouv , il est necessaire de faire une nouvelle fois la recherche
 If Trouve then Methode1; // appel recursif
end;

//***************************************//
// seconde mthode recherche les cases
// ou le candidat est seul pour la ligne , colonne ou region
//***************************************//
procedure Methode2;
Var
 I: Integer;
 Trouve : Boolean;
 Nb     : ShortInt;
begin
 Trouve := False;
 Nb :=0;
 PosteCourant := 0;
 ViderListeDecouvertes(lalisteGen,PosteCourant);
 CandidatPresentUneFois(lalisteGen,PosteCourant,Nb,Grillegen,'L');
 CandidatPresentUneFois(lalisteGen,PosteCourant,Nb,Grillegen,'C');
 CandidatPresentUneFois(lalisteGen,PosteCourant,Nb,Grillegen,'R');
 If Nb>0 then
 begin
  Trouve := True;
  for I := 0 to Nb do
   IF lalisteGen[I].Famille = 1 then
   begin
    LaCaseAjour(lalisteGen[I].Ligne,lalisteGen[I].Colonne,lalisteGen[I].Candidat);
    MettreAjourCases(lalisteGen[I].Ligne,lalisteGen[I].Colonne,lalisteGen[I].Candidat);
   end;
 end;
 If Trouve then
 begin
  Methode1;
  Methode2;
 end;
end;

procedure LaCaseAjour(Ligne,Colonne : ShortInt;Valeur : String);
Var
 I :Integer;
begin
 For I := 1 to 81 do
  If GrilleGen[I].Ligne = ligne then
   If GrilleGen[I].Colonne = Colonne then
   begin
    GrilleGen[I].Valeur  := Valeur;
    GrilleGen[I].devoile := 0;
    GrilleGen[I].Cand    := EmptyStr;
   end;
end;
end.
